unit spectrotrace;

interface

uses Windows, Classes, SysUtils, Graphics, Forms, Controls, Menus,
  Dialogs, StdCtrls, Buttons, ExtCtrls, Messages,
  clipBrd, spin, adexcept,oomisc, printers, inifiles,
  SpectroCfg, graphker, jmmUtil, Tabs, Grids, ComCtrls, ImgList,
  ToolWin, Lcd, math;

type
  TTraceSpectre = class(TForm)
    PaintBox: TPaintBox;
    ToolBar1: TToolBar;
    RegSaveBtn: TToolButton;
    SpectreBtn: TToolButton;
    ReticuleBtn: TToolButton;
    PrintBtn: TToolButton;
    ExitBtn: TToolButton;
    AbsPanel: TPanel;
    Label4: TLabel;
    CouleursPB: TPaintBox;
    ZoomBtn: TToolButton;
    BlancBtn: TToolButton;
    Label3: TLabel;
    StatusBar1: TStatusBar;
    PanelBlanc: TPanel;
    Label1: TLabel;
    ConnectBtn: TToolButton;
    SavePopupMenu: TPopupMenu;
    RegressiM: TMenuItem;
    ClipM: TMenuItem;
    FileM: TMenuItem;
    procedure ExitItemClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure PaintBoxPaint(Sender: TObject);
    procedure RegressiItemClick(Sender: TObject);
    procedure RegSaveBtnClick(Sender: TObject);
    procedure SpectreBtnClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure PrintBtnClick(Sender: TObject);
    procedure ReticuleBtnClick(Sender: TObject);
    procedure PaintBoxMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure OptionsBtnClick(Sender: TObject);
    procedure CouleursPBPaint(Sender: TObject);
    procedure ZoomBtnClick(Sender: TObject);
    procedure BlancBtnClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure PaintBoxMouseLeave(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ConnectBtnClick(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure RegressiMClick(Sender: TObject);
  private
    BlancFait : boolean;
    DejaSauve : boolean;
    PremiereFois : boolean;
    Procedure setBoutons;
    procedure FinBlanc;
    procedure FinSpectre;
    procedure avail_Lambda_Abs(Lambda : integer;abs : double);
  public
    Graphe : Tgraphe;
    procedure spectroChange;
  protected
  end;

var
  TraceSpectre: TTraceSpectre;

implementation

uses spectroDDE, spectrocst, spectromain, modebeer;

{$R *.DFM}

procedure TTraceSpectre.ExitItemClick(Sender: TObject);
begin
  Close;
end;

procedure TTraceSpectre.FormClose(Sender: TObject; var Action: TCloseAction);

procedure WriteFormPlacement;
var
  ini : TmemIniFile;
begin
  try
  Ini := TmemIniFile.create(FormMain.NomFichierIni);
  Ini.WriteInteger('WindowS','State',ord(WindowState));
  Ini.WriteInteger('WindowS','Height',height);
  Ini.WriteInteger('WindowS','Width',width);
  Ini.WriteInteger('WindowS','Top',top);
  Ini.WriteInteger('WindowS','Left',left);
  ini.updateFile;
  Ini.free;
  except
  end;
end;

begin
   writeFormPlacement;
   FormMain.setMode(mNone);
end;

procedure TTraceSpectre.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
    if (graphe.NbrePoints>0) and not(dejaSauve) and
        OKreg(stSauve)  then
          RegSaveBtnClick(sender);
    canClose := not spectroBiochrom.spectroBusy
end;

procedure TTraceSpectre.FormCreate(Sender: TObject);
begin
  Graphe := Tgraphe.Create;
  with graphe do begin
       mondeX.unite := 'nm';
       mondeX.nom := 'Lambda';
       mondeY.nom := 'A';
       mondeY.unite := '';
       mondeX.signif := stLambda;
       mondeY.signif := stAbs;
       with spectroBiochrom do setMonde(LambdaMin,0,LambdaMax,AbsMax);
       trace := trSpectre;
       canvas := PaintBox.Canvas;
  end;
  premiereFois := true;
  PaintBox.canvas.font.pitch := fpDefault;
  SetBoutons;
end;

procedure TTraceSpectre.FormDestroy(Sender: TObject);
begin
    graphe.free;
end;

procedure TTraceSpectre.FormKeyPress(Sender: TObject; var Key: Char);
begin
     if (key=' ') then
        if SpectreBtn.enabled
           then begin
              spectreBtnClick(sender);
              key := #0;
           end
           else if BlancBtn.enabled then begin
              blancBtnClick(sender);
              key := #0;
           end;
end;

procedure TTraceSpectre.PaintBoxPaint(Sender: TObject);
begin with graphe do begin
        limiteFenetre := PaintBox.clientRect;
        Draw;
        if ReticuleBtn.down then traceReticule;
end end;

procedure TTraceSpectre.RegressiItemClick(Sender: TObject);
begin
     FormDDE.Execute('FOCUS()')
end;

procedure TTraceSpectre.RegressiMClick(Sender: TObject);
begin
    regSaveBtn.ImageIndex := (sender as TmenuItem).imageIndex;
   (sender as TmenuItem).checked := true;
    regSaveBtn.caption := captionSauve[(sender as TmenuItem).tag];
    regSaveBtnClick(sender);
end;

procedure TTraceSpectre.RegSaveBtnClick(Sender: TObject);
begin
   if regressiM.Checked and
      (ConstanteDlg.showModal=mrOk) then begin
        FormDDE.EnvoieDonneesSpectre(true);
        DejaSauve := true;
        constanteDlg.newPageCB.itemIndex := 1;
   end;
   if clipM.Checked then begin
      DejaSauve := true;
      FormDDE.EnvoieDonneesSpectre(false);
   end;
   if fileM.Checked and
      FormMain.SaveDialog.execute then begin
        formDDE.saveSpectre(FormMain.saveDialog.fileName);
        DejaSauve := true;
     end;
end;

procedure TTraceSpectre.SetBoutons;
var DataOK : boolean;
begin
     dataOK := Graphe.NbrePoints>0;
     RegSaveBtn.enabled := dataOK;;
     PrintBtn.enabled := dataOK;
     spectreBtn.enabled := blancFait and
                spectroBiochrom.connecte and
                not spectroBiochrom.SpectroBusy;
     blancBtn.enabled := not spectroBiochrom.SpectroBusy and
                         spectroBiochrom.connecte;
     reticuleBtn.enabled := dataOK;
     zoomBtn.enabled := dataOK;
     label3.visible := spectroBiochrom.SpectroBusy;
     if not reticuleBtn.enabled then reticuleBtn.down := false;
     if blancFait
        then label1.Caption := aideSpectre
        else label1.Caption := aideBlanc;

end;

procedure TTraceSpectre.SpectroChange;
begin
     blancBtn.enabled := not spectroBiochrom.SpectroBusy and
                         spectroBiochrom.connecte;
     spectreBtn.enabled := blancFait and
                spectroBiochrom.connecte and
                not spectroBiochrom.SpectroBusy;
     label3.visible := spectroBiochrom.SpectroBusy;
	if spectroBioChrom.connecte
    then begin
          connectBtn.caption := stDeconnect;
          connectBtn.hint := stTouches;
          connectBtn.down := false;
          connectBtn.imageIndex := 11;
    end
    else begin
          connectBtn.caption := stConnect;
          connectBtn.hint := stRemote;
          connectBtn.down := true;
          connectBtn.imageIndex := 12;
    end
end;

procedure TTraceSpectre.SpectreBtnClick(Sender: TObject);
begin with spectroBiochrom,graphe do begin
   mondeX.mini := LambdaMin;
   mondeX.maxi := LambdaMax;
   limiteFenetre := PaintBox.clientRect;
   lanceSpectre(finSpectre,avail_lambda_abs);
   label3.caption := stAcqSpectre;
   SetBoutons;
   NbrePoints := 0;
   PaintBox.refresh;
end end;

procedure TTraceSpectre.BlancBtnClick(Sender: TObject);
begin
   spectroBiochrom.lanceZero(finBlanc);
   label3.caption := stBlanc;
   SetBoutons;
   graphe.NbrePoints := 0;
   PaintBox.refresh;
end;

procedure TTraceSpectre.ZoomBtnClick(Sender: TObject);

procedure Zoom;
var j : integer;
    max : double;
begin with graphe do begin
    max := 0;
    for j := 0 to pred(NbrePoints) do
        if data.valeurY[j]>max then max := data.valeurY[j];
    max := max*1.05;
    if max>spectroBiochrom.absMax then max := spectroBiochrom.absMax;
    if max<0.2 then max := 0.2;
    mondeY.maxi := max;
    paintBox.refresh;
end end;

procedure Total;
begin with graphe do begin
    mondeY.maxi := spectroBiochrom.AbsMax;
    mondeX.maxi := spectroBiochrom.LambdaMax;
    mondeX.mini := spectroBiochrom.LambdaMin;
    paintBox.refresh;
end end;

begin
    if zoomBtn.down then zoom else total;
end;

procedure TTraceSpectre.OptionsBtnClick(Sender: TObject);
begin
     ModeBeerDlg := TModeBeerDlg.create(self);
     ModeBeerDlg.showModal;
     ModeBeerDlg.free;
end;

procedure TTraceSpectre.FormActivate(Sender: TObject);
var ini : TiniFile;
begin
     if premiereFois then begin
        if FileExists(FormMain.NomFichierIni) then begin
        try
        Ini := TIniFile.create(FormMain.NomFichierIni);
        WindowState := TwindowState(Ini.ReadInteger('WindowS','State',ord(wsMaximized)));
        Height := Ini.ReadInteger('WindowS','Height',height);
        Width := Ini.ReadInteger('WindowS','Width',width);
        Top := Ini.ReadInteger('WindowS','Top',top);
        Left := Ini.ReadInteger('WindowS','Left',left);
        Ini.free;
        except
        end;
        end;
     end; // premiereFois
     premiereFois := false;
     Label4.Caption := '';
     with spectroBiochrom do
          graphe.setMonde(LambdaMin,0,LambdaMax,AbsMax);
     PaintBox.Refresh;
     SetBoutons;
end;

procedure TTraceSpectre.PrintBtnClick(Sender: TObject);
var marge : integer;
begin with Printer do begin
    BeginDoc; // dbut de l'impression
    graphe.canvas := canvas;
    graphe.limiteFenetre := Canvas.ClipRect;
    with graphe.limiteFenetre do begin
         marge := (right-left) div 20;
         right := right-marge;
         left := left+marge;
         if printer.orientation=poPortrait then
             bottom := (top+bottom) div 2;
// portrait => moiti haut ; paysage => total 
         marge := (bottom-top) div 20;
         bottom := bottom - marge;
         top := top + marge;
    end;
    graphe.Draw;
    EndDoc; { fin de l'impression }
    graphe.canvas := paintBox.canvas;
end end;

procedure TTraceSpectre.ReticuleBtnClick(Sender: TObject);
begin
    graphe.TraceReticule;
    setBoutons;
end;

procedure TTraceSpectre.PaintBoxMouseLeave(Sender: TObject);
begin
	label4.Caption := '';
end;

procedure TTraceSpectre.PaintBoxMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
var Xr,Yr,pente : double;
    j : integer;
begin
   if ReticuleBtn.down then with Graphe do begin
      TraceReticule; { efface }
      MondeXY(x,y,xr,yr);
      if (Xr>=valeurX[0]) and (Xr<=valeurX[pred(NbrePoints)])
         then begin
            j := 0;
            while xr>valeurX[j] do inc(j);
            if j=0 then yr := data.valeurY[0]
            else begin
               pente := (data.valeurY[j]-data.valeurY[j-1])/(valeurX[j]-valeurX[j-1]);
               Yr := data.valeurY[j]+(Xr-valeurX[j])*pente;
            end;
            windowXY(xr,yr,x,y);
         end;
      YReticule := Y;
      Label4.caption := 'Lambda='+IntTostr(round(Xr))+
                        ' nm ; A= '+
                        spectroBiochrom.FormatAbs(Yr);
      XReticule := X;
      TraceReticule; { marque }
end end;

procedure TTraceSpectre.ConnectBtnClick(Sender: TObject);
begin
	if spectroBioChrom.connecte
       then spectroBiochrom.release
       else spectroBiochrom.control;
end;

procedure TTraceSpectre.CouleursPBPaint(Sender: TObject);
var i,lambda : integer;
    A : double;
    couleur : Tcolor;
begin
    A := (graphe.mondeX.maxi-graphe.mondeX.mini) /
         (graphe.limiteCourbe.right-graphe.limiteCourbe.left);
    for i := 0 to CouleursPB.Width-1 do begin
       lambda := round(graphe.mondeX.mini + A*(i-graphe.limiteCourbe.left));
       couleur := WaveLengthToColor(lambda);
       CouleursPB.canvas.pen.color := couleur;
       CouleursPB.canvas.moveTo(i,0);
       CouleursPB.canvas.lineTo(i,CouleursPB.Height-1);
    end;
end;

procedure TTraceSpectre.Avail_Lambda_Abs(Lambda : integer;abs : double);
begin
     try
     with graphe do begin
          if nbrePoints=0 then label3.caption := stLectureSpectre;
          valeurX[NbrePoints] := lambda;
          data.valeurY[NbrePoints] := abs;
          traceUnPoint(NbrePoints,data);
          inc(NbrePoints);
          if data.valeurY[NbrePoints]>mondeY.maxi then begin
             mondeY.maxi := data.valeurY[NbrePoints]*1.05;
             paintBox.refresh;
          end;
          DejaSauve := false;
     end;
     except
     end;
end;

procedure TTraceSpectre.FinBlanc;
begin
    BlancFait := true;
    SetBoutons;
end;

procedure TTraceSpectre.FinSpectre;
begin
    SetBoutons;
end;

end.

