unit spectromain;

interface

uses Windows, Classes, Graphics, Forms, Controls, Menus,
     Dialogs, StdCtrls, Buttons, ExtCtrls, sysUtils, Messages,
     clipBrd, spin, registry, ComCtrls, XPMan, inifiles,
     spectrocst, jmmUtil, spectrocfg, tradformU,
     AwWin32, ADport, adexcept, oomisc, awuser, oneinst,
     IvDictio, IvAMulti, IvFiMult, IvMulti, ImgList;

type
  Tmode = (mNone,mSpectre,mCinetique,mBeer);

  TFormMain = class(TTradForm)
    VoieSerie: TApdComPort;
    BoutonPanel: TPanel;
    ExitBtn: TBitBtn;
    AboutBtn: TBitBtn;
    CinetiqueBtn: TBitBtn;
    SpectreMinMaxBtn: TBitBtn;
    BeerBtn: TBitBtn;
    XPManifest1: TXPManifest;
    ComBtn: TBitBtn;
    ComPorts: TComboBox;
    ChercheLabel: TLabel;
    ConnectBtn: TBitBtn;
    Dictionary: TIvTextDictionary;
    StatusBar1: TStatusBar;
    ImageList1: TImageList;
    SaveDialog: TSaveDialog;
    procedure ExitItemClick(Sender: TObject);
    procedure AboutItemClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure CinetiqueBtnClick(Sender: TObject);
    procedure SpectreMinMaxBtnClick(Sender: TObject);
    procedure BeerBtnClick(Sender: TObject);
    procedure ComBtnClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure ConnectBtnClick(Sender: TObject);
    procedure LangueRGClick(Sender: TObject);
  private
    spectroReady : boolean;
    numTest : integer;
    Procedure AppActivate(sender : Tobject);
    Procedure ShowHint(Sender: TObject);
    procedure setFocusMain;
  public
    AcquisitionEnCours : Tmode;
    nomFichierIni : string;
    Procedure SetMode(Acq : Tmode);
    Procedure Erreur(Erreur : shortString);
    Procedure SpectroChange;
  protected
    procedure WMFocus(var Msg: Tmessage); message wm_Focus;
  end;

var
  FormMain: TFormMain;

implementation

uses about, SpectroCinet, SpectroBeer, SpectroTrace;

{$R *.DFM}

const maxNumTest = 5;

Procedure TformMain.AppActivate(sender : Tobject);
begin
    Case AcquisitionEnCours of
       mSpectre : TraceSpectre.setFocus;
       mCinetique : CinetiqueForm.setFocus;
       mBeer : BeerForm.setFocus;
    end;
end;

procedure TFormMain.ExitItemClick(Sender: TObject);
begin
   Close
end;

procedure TFormMain.AboutItemClick(Sender: TObject);
begin
    AboutBox.ShowModal
end;

procedure TFormMain.FormCreate(Sender: TObject);
var
   ini: tIniFile;
begin
  Application.OnActivate := AppActivate;
  Application.OnHint := ShowHint;
  Application.ShowHint := true;
  AcquisitionEnCours := mNone;
  spectroBiochrom.voieSerie := voieSerie;
  NomFichierIni := changeFileExt(application.exename,'.INI');
  if FileExists(NomFichierIni) then begin
     Ini := TIniFile.Create(NomFichierIni);
     numTest := Ini.ReadInteger('Spectro', 'Test', 1);
     ini.free;
  end;
end;

procedure TFormMain.LangueRGClick(Sender: TObject);
begin
	// dictionary.language := langueRG.itemIndex-2;
    translator.translate;
end;

procedure TFormMain.FormActivate(Sender: TObject);
begin
    spectroBiochrom.FonErreur := Erreur;
    spectroBiochrom.FonChange := SpectroChange;
    setMode(mNone);
end;

procedure TFormMain.FormClose(Sender: TObject;var Action: TCloseAction);
var
   ini: tIniFile;
begin
    if voieSerie.open then begin
       if spectroBiochrom.connecte then spectroBioChrom.release;
       spectroBiochrom.voieSerie.onTriggerAvail := nil;
       if VoieSerie.Tracing=tlOn then VoieSerie.Tracing := tlDump;
       if VoieSerie.Logging=tlOn then VoieSerie.Logging := tlDump;
       ini := TIniFile.Create(NomFichierIni);
       Ini.WriteInteger('Spectro', 'Test', numTest);
       ini.free;
    end;
end;

procedure TFormMain.SetMode(Acq : Tmode);               

Procedure Initialiser;

Procedure Test(numero : integer);
var versionStr : shortString;
begin
{$IFDEF DEBUG}
     VoieSerie.traceName := mesDocsDir+'Biochrom'+intTostr(numTest)+'.trc';
     VoieSerie.logName := mesDocsDir+'Biochrom'+intTostr(numTest)+'.log';
     VoieSerie.Tracing := tlOn;
     VoieSerie.Logging := tlOn;
     inc(numTest);
     if numTest>maxNumTest then numTest := 0;
{$ENDIF}
     spectroReady := false;
     ChercheLabel.caption := stRechercheCom+intTostr(numero)+' ...';
     try
     voieSerie.TriggerLength := 1;
     voieSerie.comNumber := numero;
     voieSerie.Open := true;
     versionStr := spectroBiochrom.Version;
     if versionStr<>''
       then begin
          spectroBiochrom.ComNumber := numero;
          spectroReady := true;
       end
       else begin
          versionStr := '';
          spectroReady := false;
          voieSerie.Open := false;
       end;   
     except
         spectroReady := false;
         voieSerie.Open := false;
     end;
end;

var i : integer;
begin
     if isPortAvailable(spectroBiochrom.ComNumber) then
        test(spectroBiochrom.ComNumber);
     if not VoieSerie.open then
     for i := 1 to MaxComHandles do begin
          if isPortAvailable(i) and
            (i<>spectroBiochrom.ComNumber) then test(i);
          if VoieSerie.open then break;
     end;
     if VoieSerie.open
     then begin
        caption := 'Acquisition par'+' '+spectroBiochrom.nom+' '+'pour Regressi';
        if spectroBiochrom.control then begin
           connectBtn.caption := stDeconnect;
           connectBtn.hint := stTouches;
           statusBar1.simpleText := '';
        end
        else begin
           showMessage(spectroBiochrom.codeErreur);
           if spectroBiochrom.control then begin
              connectBtn.caption := stDeconnection;
              connectBtn.hint := stTouches;
           end
           else begin
              connectBtn.caption := stConnection;
              connectBtn.hint := stRemote;
              statusBar1.simpleText := spectroBiochrom.codeErreur;
           end;
        end;
        CinetiqueBtn.visible := spectroReady and spectroBiochrom.connecte;
        BeerBtn.visible := spectroReady and spectroBiochrom.connecte;
        SpectreMinMaxBtn.visible := spectroReady and spectroBiochrom.connecte;
        ConnectBtn.visible := true;
     end
     else begin
        caption := stNoSpectro;
        afficheErreur(caption);
        ComPorts.visible := Comports.items.count>1;
        if ComPorts.visible
           then ComBtn.hint := stRecommencer+' avec la voie srie ci-dessous'
           else ComBtn.hint := stRecommencer;
        ConnectBtn.visible := false;
     end;
end; // initialiser

Procedure SetComPorts;
var i,j : integer;
begin
    ComPorts.items.clear;
    for i := 1 to MaxComHandles do
        if IsPortAvailable(i) then begin
           ComPorts.Items.Add('COM'+IntToStr(i));
           j := ComPorts.Items.count-1;
           if j=spectroBiochrom.ComNumber then ComPorts.itemIndex := j;
         end;
    if ComPorts.itemIndex<0 then ComPorts.itemIndex := 0;
end;

begin
     constanteDlg.newPageCB.itemIndex := 0;
     if not VoieSerie.open then begin
        ChercheLabel.caption := stRecherche;
        ChercheLabel.visible := true;
        SetComPorts;
        Initialiser;
     end;
     CinetiqueBtn.visible := false;
     BeerBtn.visible := false;
     SpectreMinMaxBtn.visible := false;
     ConnectBtn.visible := false;
     ChercheLabel.visible := false;
     if not VoieSerie.open then begin
        exit;
     end;
     if spectroReady then begin
          AcquisitionEnCours := Acq;
          CinetiqueBtn.visible := spectroBioChrom.connecte and (Acq in [mNone,mCinetique]);
          BeerBtn.visible := spectroBioChrom.connecte and (Acq in [mNone,mBeer]);
          SpectreMinMaxBtn.visible := spectroBioChrom.connecte and (Acq in [mNone,mSpectre]);
          ConnectBtn.visible := true;
          case Acq of
             mBeer : BeerForm.show;
             mCinetique : CinetiqueForm.show;
             mSpectre : TraceSpectre.show;
             mNone : ;
         end;
      end
      else ConnectBtn.visible := false;
      ComPorts.visible := false;
end; // setMode

procedure TFormMain.ShowHint(Sender: TObject);
begin
      case AcquisitionEnCours of
          mSpectre : TraceSpectre.statusBar1.SimpleText := Application.Hint;
          mCinetique : CinetiqueForm.statusBar1.SimpleText := Application.Hint;
          mBeer : BeerForm.statusBar1.SimpleText := Application.Hint;
          mNone : statusBar1.SimpleText := Application.Hint;
      end
end;

procedure TFormMain.CinetiqueBtnClick(Sender: TObject);
begin
     setMode(mCinetique)
end;

procedure TFormMain.ComBtnClick(Sender: TObject);
begin
     spectroBiochrom.ComNumber :=
         strToInt(copy(Comports.items[ComPorts.itemIndex],
             4,length(Comports.items[ComPorts.itemIndex])-3));
     setMode(mNone);
end;

procedure TFormMain.ConnectBtnClick(Sender: TObject);
begin
	if spectroBiochrom.connecte
       then begin
          spectroBiochrom.release;
          connectBtn.caption := stConnection;
          connectBtn.hint := stRemote;
          statusBar1.simpleText := '';
       end
       else begin
           spectroBiochrom.control;
           connectBtn.caption := stDeconnection;
           connectBtn.hint := stTouches;
       end;
    CinetiqueBtn.visible := spectroReady and spectroBiochrom.connecte;
    BeerBtn.visible := spectroReady and spectroBiochrom.connecte;
    SpectreMinMaxBtn.visible := spectroReady and spectroBiochrom.connecte;
end;

procedure TFormMain.SpectreMinMaxBtnClick(Sender: TObject);
begin
    setMode(mSpectre)
end;

procedure TFormMain.BeerBtnClick(Sender: TObject);
begin
    setMode(mBeer)
end;

procedure TFormMain.setFocusMain;
begin
     case AcquisitionEnCours of
        mSpectre :  if TraceSpectre=nil then AcquisitionEnCours := mNone;
        mCinetique : if CinetiqueForm=nil then AcquisitionEnCours := mNone;
        mBeer : if BeerForm=nil then AcquisitionEnCours := mNone;
     end;
     case AcquisitionEnCours of
        mSpectre : begin
              TraceSpectre.show;
              TraceSpectre.setFocus;
        end;
        mCinetique : begin
            CinetiqueForm.show;
            CinetiqueForm.setFocus;
        end;
        mBeer : begin
            BeerForm.show;
            BeerForm.setFocus;
        end;
        else setFocus
     end;
end;

procedure TFormMain.WMFocus(var Msg: Tmessage);
begin
   setFocusMain;
   Msg.result := 0;
end;

Procedure TFormMain.Erreur(Erreur : shortString);
begin
    case AcquisitionEnCours of
        mSpectre : TraceSpectre.statusBar1.panels[1].text := erreur;
        mCinetique : CinetiqueForm.statusBar1.panels[1].text := erreur;
        mBeer : BeerForm.statusBar1.panels[1].text := erreur;
        else statusBar1.simpleText := erreur;
     end;
end;

Procedure TFormMain.SpectroChange;
begin
    case AcquisitionEnCours of
        mSpectre : TraceSpectre.spectroChange;
        mCinetique : CinetiqueForm.spectroChange;
        mBeer : BeerForm.spectroChange;
        else ;
     end;
end;

end.
