unit graphker;

interface

uses windows, SysUtils, Graphics, Classes, Printers,
   Forms, Controls, Buttons, StdCtrls, ExtCtrls, Dialogs, Math,
   jmmutil, Spectrocfg;

const
   IntSansSignif   = -1;
   CouleurCalc     = clRed;
   CouleurTitre    = clBlack;
   CouleurAxe      = clBlack;
   CouleurSpectre  = clBlue;
   Precision       = 7;
   LongNombreAxe   = 4;
   NbreGraduations = 10;
   NbreGradMin     = 5;

type
   TEtatSpectre   = (esBlancEnCours, esSpectreEnCours, esAttente);
   TEtatCinetique = (eaAttenteBlanc, eaEnCours, eaTerminee, eaAttente, eaSpectreEnCours);
   TCurseurCinetique = (curNone,curLambda,curLambdaBis);
   Ttrace         = (trLigne, trSpectre, trPoint, trLignePoint);

   Tmonde = class
      mini, maxi: double;
      a, b:       double; { entier = rel*a+b }
      zeroInclus: boolean;
      avecAxe, grandAxe: boolean;
      exposant : double;
      nom:        strCourte;
      unite:      strCourte;
      signif:     shortString;
      PosCurseur: integer;
      deltaAxe, pointDebut: double;
      NbreChiffre : integer;
      // deltaAxe = cart entre graduations sur les axes
      constructor Create;
      procedure setDelta;
      procedure affecteEntier(r: double; var i: integer);
      function FormatAxe(x: double): strCourte;
      Function NomAxe : strCourte;
   end;

   Tcourbe = class
      couleur: Tcolor;
      valeurY:  vecteur;
      titre : shortString;
   end;

   Tgraphe = class
      dimPoint:       integer;
      mondeX, mondeY: Tmonde;
      LimiteCourbe, LimiteFenetre: Trect;
      LargCarac, HautCarac, Marge: integer;
      quadrillage:    boolean;
      canvas:         Tcanvas;
      XReticule, YReticule: integer;
      Data,DataBis:   Tcourbe;
      NbrePoints: indiceVecteur;
      valeurX:     vecteur;
      Trace:      Ttrace;
      BisActif : boolean;
      avecAxeY : boolean;
      constructor Create;
      destructor Destroy; override;
      procedure Draw;
      function MaxiX: double;
      function MaxiY: double;
      procedure Tri;
      procedure WindowXY(xr, yr: double; var xi, yi: integer);
      procedure MondeXY(xi, yi: integer; var xr, yr: double);
      procedure ChercheMonde;
      procedure SetMonde(minX, minY, maxX, maxY: double);
      procedure DrawAxis;
      procedure TraceUnPoint(i: indiceVecteur;d : Tcourbe);
      procedure SetHauteurFonte(H: integer);
      procedure TraceReticule;
      procedure TraceCroix(x, y : integer);
      procedure TracePointCalc(x, y: double);
      procedure Imprime;
   end;

implementation

constructor Tmonde.Create;
begin
   inherited Create;
   A := 1;
   B := 0;
   nom := '';
   unite := '';
   signif := '';
   ZeroInclus := True;
   Maxi := 255;
   Mini := 0;
   PosCurseur := intSansSignif;
end; // Tmonde.Create 

procedure Tmonde.affecteEntier(r: double; var i: integer);
begin
   if r = sansSignif
      then i := intSansSignif
      else i := round(A * r + B);
end;

procedure TGraphe.MondeXY(xi, yi: integer; var xr, yr: double);
begin
   xr := (xi - mondeX.B) / mondeX.A;
   yr := (yi - mondeY.B) / mondeY.A;
end;

procedure Tgraphe.SetHauteurFonte(H: integer);
begin
   with canvas.Font do begin
      if H <= 13 then
         H := 13 { 8 points = 13 pixels }
      else if H <= 18 then
         H := 16 { 10 points = 16 pixels }
      else if H <= 24 then
         H := 20; { 12 points = 20 pixels }
      Name := 'Tahoma';
      Height := H;
      HautCarac := canvas.TextHeight('A');
      Marge := hautCarac div 3;
      LargCarac := canvas.TextWidth('A');
   end;
end;

procedure TGraphe.WindowXY(xr, yr: double; var xi, yi: integer);
begin
    mondeX.affecteEntier(xr, xi);
    mondeY.affecteEntier(yr, yi);
end;

procedure TGraphe.ChercheMonde;

   procedure chercheProjectionY;
   begin with mondeY do begin
         A := (limiteCourbe.Top - limiteCourbe.Bottom) / (Maxi - Mini);
         B := limiteCourbe.Bottom - Mini * A;
         setDelta;
   end end; { chercheProjectionY }

   procedure chercheProjectionX;
   begin with mondeX do begin
         A := (limiteCourbe.right - limiteCourbe.left) / (Maxi - Mini);
         B := limiteCourbe.left - Mini * A;
         setDelta;
   end end;

begin // ChercheMonde 
   with limiteFenetre do begin
      mondeY.avecAxe := avecAxeY and
         ((bottom - top) > NbreGradMin * hautCarac * 2);
      mondeX.avecAxe := (right - left) > NbreGradMin * longNombreAxe * largCarac;
      mondeY.grandAxe := (bottom - top) > NbreGraduations * hautCarac * 2;
      mondeX.grandAxe := (right - left) > NbreGraduations * longNombreAxe * largCarac;
   end;
   limiteCourbe := limiteFenetre;
   Inc(limiteCourbe.left, marge);
   Inc(limiteCourbe.top, marge);
   Dec(limiteCourbe.bottom, marge);
   Dec(limiteCourbe.right, marge);
   if mondeY.avecAxe then
      Inc(limiteCourbe.left, LongNombreAxe * largCarac);
   if mondeY.avecAxe or not avecAxeY then
      Inc(limiteCourbe.top, hautCarac);
   if mondeX.avecAxe or not avecAxeY then
      Dec(limiteCourbe.bottom, hautCarac);
   chercheProjectionX;
   chercheProjectionY;
end; { ChercheMonde }

procedure TGraphe.SetMonde(minX, minY, maxX, maxY: double);
begin
   mondeX.mini := minx;
   mondeX.maxi := maxx;
   mondeY.mini := miny;
   mondeY.maxi := maxy;
end; { SetMonde }

procedure Tgraphe.TraceUnPoint(i: indiceVecteur;d : Tcourbe);
var
   xi, yi, x0, y0: integer;
begin with D do begin
      canvas.pen.color := Couleur;
      canvas.pen.style := psSolid;
      WindowXY(valeurX[i], valeurY[i], xi, yi);
      if (xi = IntSansSignif) or (yi = IntSansSignif) then
         exit;
      canvas.pen.Width := 1;
      if trace in [trPoint, trLignePoint] then
         traceCroix(xi, yi);
      if (trace = trPoint) or (i = 0) then
         exit;
      dec(i);
      WindowXY(valeurX[i], valeurY[i], x0, y0);
      if (x0 <> IntSansSignif) and (y0 <> IntSansSignif) then
         with canvas do begin
            moveTo(x0, y0);
            lineTo(xi, yi);
         end;
end end;// traceUnPoint

procedure TGraphe.Draw;

procedure TraceLigne;
var
   i: integer;
   PointPrecedent: boolean;
   xi, yi: integer;
begin
   if NbrePoints = 0 then exit;
   with canvas, Data do begin
      pen.style := psSolid;
      pen.color := couleur;
      pen.Width := 1;
      PointPrecedent := False;
      for i := 0 to pred(NbrePoints) do begin
         WindowXY(valeurX[i], valeurY[i], xi, yi);
         if (xi = IntSansSignif) or (yi = IntSansSignif) then
            pointPrecedent := False
         else if PointPrecedent then
            LineTo(xi, yi)
         else begin
            MoveTo(xi, yi);
            pointPrecedent := True;
         end;
      end;{for i}
   end;
   if bisActif then with canvas, DataBis do begin
      pen.color := couleur;
      PointPrecedent := False;
      for i := 0 to pred(NbrePoints) do begin
         WindowXY(valeurX[i], valeurY[i], xi, yi);
         if (xi = IntSansSignif) or (yi = IntSansSignif) then
            pointPrecedent := False
         else if PointPrecedent then
            LineTo(xi, yi)
         else begin
            MoveTo(xi, yi);
            pointPrecedent := True;
         end;
      end;{for i}
   end;
end;// TraceLigne 

procedure TracePoints;
var
   i: integer;
   xi, yi: integer;
begin
   if NbrePoints = 0 then exit;
   with canvas, Data do begin
      pen.style := psSolid;
      pen.color := couleur;
      pen.Width := 1;
      for i := 0 to pred(NbrePoints) do  begin
         WindowXY(valeurX[i], valeurY[i], xi, yi);
         if (xi <> IntSansSignif) and (yi <> IntSansSignif) then
            TraceCroix(xi, yi);
      end;
   end;
   if BisActif then with canvas, DataBis do begin
      pen.color := couleur;
      for i := 0 to pred(NbrePoints) do  begin
         WindowXY(valeurX[i], valeurY[i], xi, yi);
         if (xi <> IntSansSignif) and (yi <> IntSansSignif) then
            TraceCroix(xi, yi);
      end;
   end;
end;// TracePoints

procedure TraceSpectre(d : Tcourbe);
var
   i: integer;
   xi, yi: integer;
begin with canvas, d do begin
      if NbrePoints=0 then exit;
      pen.style := psSolid;
      pen.color := couleur;
      pen.width := 1;
      WindowXY(valeurX[0], valeurY[0], xi, yi);
      MoveTo(xi, yi);
      for i := 0 to pred(NbrePoints) do begin
         WindowXY(valeurX[i], valeurY[i], xi, yi);
         LineTo(xi, yi)
      end;{for i}
end end;{ TraceSpectre }

begin
   with canvas do begin
      with limiteFenetre do begin
         dimPoint := (bottom - top) div 150;
         if dimPoint = 0 then
            dimPoint := 1;
         setHauteurFonte((bottom - top) div 30);
      end;
      pen.style := psDot;
      pen.color := CouleurAxe;
      brush.style := bsSolid;
      brush.color := clWhite;
      FillRect(LimiteFenetre);
      chercheMonde;
      drawAxis;
      case Trace of
         trSpectre: traceSpectre(data);
         trLigne: traceLigne;
         trPoint: tracePoints;
         trLignePoint: begin
            traceLigne;
            tracePoints;
         end;
      end;
   end;
end; // draw

constructor TGraphe.Create;
begin
   inherited Create;
   mondeX := Tmonde.Create;
   mondeY := Tmonde.Create;
   quadrillage := True;
   Trace := trLigne;
   Data := Tcourbe.Create;
   NbrePoints := 0;
   Data.couleur := clBlue;
   DataBis := Tcourbe.Create;
   DataBis.couleur := clBlue;
   avecAxeY := true;
end;

destructor TGraphe.Destroy;
begin
   mondeX.Free;
   mondeY.Free;
   Data.Free;
   inherited Destroy;
end;

procedure Tmonde.SetDelta;
{ Exposant=puissance de 10 multiple de 3 de manire  avoir entre
NbreGraduations (10) et N/2.5 (4) graduations sur l'axes.
ou NbreGradMin (5) et N/2.5 (2) graduations sur l'axes.
Delta est l'intervalle entre les graduations qui commencent  PremiereGraduation }
const
   coeff: array[0..5] of double = (0.1, 0.2, 0.5, 1, 2, 5);
var
   largeur: double;
   Exponent,N: integer;
   index: integer;
begin
   Exponent := floor(Log10(Maxi - Mini));
   largeur := (maxi - mini) * intPower(10, -exponent);
   index := 0;
   if grandAxe
      then  while ((largeur / coeff[index]) >= NbreGraduations) and
                  (index < 5) do
         Inc(index)
      else  while ((largeur / coeff[index]) >= NbreGradMin) and
                (index < 5) do
         Inc(index);
   DeltaAxe := coeff[index] * intPower(10, exponent);
   PointDebut := ceil(mini / deltaAxe) * deltaAxe;
   Index := Exponent div 3;
   if ((Exponent mod 3) < 0) then
      Dec(index);
// la division euclidienne des informaticiens <> div des maths ! 
   if index=0
      then Exposant := 1
      else Exposant := power(10, 3 * Index);
   N := ceil(Log10(Maxi / Exposant));
   if N > NbreChiffre then
      NbreChiffre := N;
   if NbreChiffre < 3 then
      NbreChiffre := 3;
   if (NbreChiffre > longNombreAxe) then
      NbreChiffre := longNombreAxe;
end;

function Tmonde.FormatAxe(x: double): strCourte;
begin
   FormatAxe := FloatToStrF(x/exposant, ffGeneral, NbreChiffre, 0);
end;

procedure Tgraphe.TraceReticule;
begin
   with canvas do begin
      Pen.mode  := pmNotXor;
      Pen.color := clBlack;
      MoveTo(xReticule, clipRect.top);
      LineTo(xReticule, clipRect.bottom);
      MoveTo(clipRect.left, yReticule);
      LineTo(clipRect.right, yReticule);
      Pen.mode := pmCopy;
   end;
end;

procedure Tgraphe.TracePointCalc(x, y: double);
var
   xi, yi: integer;
begin
   canvas.Pen.color := couleurCalc;
   canvas.pen.style := psSolid;
   WindowXY(x, y, xi, yi);
   traceCroix(xi, yi);
   canvas.pen.Width := 1;
end;// TracePointCalc

function Tgraphe.MaxiY: double;
var
   j: integer;
begin
   Result := 0;
   for j := 0 to pred(NbrePoints) do
      if data.valeurY[j] > Result then
         Result := data.valeurY[j];
end; // maxiY

function Tgraphe.MaxiX: double;
var
   j: integer;
begin
   Result := 0;
   for j := 0 to pred(NbrePoints) do
      if valeurX[j] > Result then
         Result := valeurX[j];
end; // maxiX

procedure Tgraphe.TraceCroix(x, y : integer);
begin
     canvas.Brush.Style := bsSolid;
     canvas.Brush.color := canvas.pen.color;
     canvas.ellipse(x-dimPoint, y - dimPoint, x+dimPoint, y + dimPoint);
     canvas.Brush.Style := bsClear;     
end;// TraceCroix

procedure Tgraphe.Imprime;
var
   sauveCanvas: Tcanvas;
begin
   if MessageDlg('Impression ?', mtConfirmation, mbOKCancel, 0) = mrOk then begin
      sauveCanvas := canvas;
      printer.Orientation := poLandscape;
      printer.beginDoc;
      canvas := printer.canvas;
      limiteFenetre := canvas.clipRect;
      draw;
      printer.endDoc;
      canvas := SauveCanvas;
   end;
end;

procedure TGraphe.DrawAxis;
var
   xAxeY, yAxeX: integer;
   xNombreMax, signeMargeX: integer;
   PointCourant: single;

   procedure TitreAxeX;
   var
      xTitre, yTitre: integer;
      prompt: string;
   begin
      yTitre := limiteCourbe.bottom + marge;
      prompt := mondeX.nomAxe;
      xTitre := LimiteFenetre.right - canvas.textWidth(prompt);
      canvas.TextOut(xTitre, yTitre, prompt);
      xNombreMax := xTitre - largCarac * longNombreAxe div 2;
      if xtitre < xNombreMax then
         xNombreMax := xTitre;
   end;

   procedure TitreAxeY;
   var
      prompt: string;
   begin
      prompt := mondeY.nomAxe;
      canvas.textOut(limiteFenetre.left + marge, limiteFenetre.Top, prompt);
   end;

   function YcorrectNombre(y: integer): boolean;
   var
      limiteTop: integer;
   begin with mondeY do begin
         LimiteTop := limiteCourbe.Top + marge;
         YcorrectNombre := (y > (LimiteTop)) and (y <
            (limiteCourbe.Bottom - marge)) and (abs(y - yAxeX) > marge);
   end end;

   function YcorrectGrad(y: integer): boolean;
   begin with mondeY do
         YcorrectGrad := (y > limiteCourbe.Top) and
            (y < limiteCourbe.Bottom);
   end;

   function XcorrectNombre(x: integer): boolean;
   begin
      XcorrectNombre := (x > (limiteCourbe.left + marge)) and
         (x < xNombreMax) and (abs(x - xAxeY) > marge);
   end;

   function XcorrectGrad(x: integer): boolean;
   begin
      XcorrectGrad := (x > limiteCourbe.left) and
         (x < limiteCourbe.right);
   end;

   procedure OutTextY(y: integer; S: string);
   begin
      canvas.TextOut(limiteFenetre.left + marge, y - hautCarac div 2, S);
   end;

   procedure OutTextX(x: integer; S: string);
   begin
      canvas.TextOut(x-(canvas.TextWidth(S) div 2), yAxeX + marge, S);
   end;

   procedure MyLine(x1, y1, x2, y2: integer);
   begin
      canvas.moveTo(x1, y1);
      canvas.lineTo(x2, y2);
   end;

   procedure graduationX(x: integer; quadrillage: boolean);
   begin
      if quadrillage
         then MyLine(x, limiteCourbe.top, x, limiteCourbe.bottom)
         else MyLine(x, yAxeX, x, yAxeX + marge);
   end;

   procedure graduationY(y: integer; quadrillage: boolean);
   begin
      if quadrillage
         then MyLine(limiteCourbe.left, y, limiteCourbe.right, y)
         else MyLine(xAxeY, y, xAxeY - signeMargeX * marge, y);
   end;

   procedure initAxeX;
   var
      x1, y1: integer;
   begin with mondeX, canvas do begin
         PointCourant := PointDebut;
         WindowXY(0, 0, x1, y1);
         if (x1 > limiteCourbe.left) and
            (x1 < limiteCourbe.right) then begin  // zro
            pen.style := psDot;
            Myline(x1, limiteCourbe.bottom, x1, limiteCourbe.top);
            pen.style := psSolid;
         end;
         TitreAxeX;
         MyLine(limiteFenetre.left, yAxeX, limiteCourbe.right, yAxeX);
         // flches
         MyLine(LimiteCourbe.right - marge * 3, yAxeX + marge, LimiteCourbe.right, yAxeX);
         MyLine(LimiteCourbe.right - marge * 3, yAxeX - marge, LimiteCourbe.right, yAxeX);
      end;
   end; // initAxeX

   procedure traceAxeX;
   var
      xs, ys: integer;
      NombreAff: strCourte;
   begin with mondeX do begin
         initAxeX;
         if Quadrillage then
            canvas.pen.style := psDot;
         repeat
            windowXY(PointCourant, mondeY.Mini, xs, ys);
            if XcorrectGrad(xs) then begin
               graduationX(xs, Quadrillage);
               if XcorrectNombre(xs) then
               begin
                  if abs(PointCourant) < (Maxi - Mini) / 100 then
                     pointCourant := 0;
                  NombreAff := formatAxe(pointCourant);
                  OutTextX(xs, NombreAff);
               end;
            end;
            PointCourant := PointCourant + deltaAxe;
         until (PointCourant >= Maxi);
         canvas.pen.style := psSolid;
   end end;{traceAxeX}

   procedure initAxeY;
   var
      x1, y1: integer;
   begin with mondeY do begin
         PointCourant := PointDebut;
         WindowXY(0, 0, x1, y1);
         if (y1 > limiteCourbe.Top) and
            (y1 < limiteCourbe.Bottom) then begin  // zro
            canvas.pen.style := psDot;
            Myline(limiteCourbe.left, y1, limiteCourbe.right, y1);
            canvas.pen.style := psSolid;
         end;
         MyLine(xAxeY, yAxeX, xAxeY, limiteCourbe.Top - marge);
         { flches }
         MyLine(xAxeY - marge, LimiteCourbe.Top + 2 * marge, xAxeY, LimiteCourbe.Top - marge);
         MyLine(xAxeY + marge, LimiteCourbe.Top + 2 * marge, xAxeY, LimiteCourbe.Top - marge);
         TitreAxeY;
   end  end; // initAxeY

   procedure traceAxeY;
   var
      xs, ys: integer;
      nombreAff: strCourte;
   begin with mondeY do begin
         initAxeY;
         if Quadrillage then
            canvas.pen.style := psDot;
         repeat
            windowXY(mondeX.Mini, PointCourant, xs, ys);
            // coordonne cran de marque
            if YcorrectGrad(ys) then begin
               graduationY(ys, quadrillage);
               if YcorrectNombre(ys) then begin
                  if abs(pointCourant) < (Maxi - Mini) / 1000 then
                     pointCourant := 0;
                  NombreAff := formatAxe(pointCourant);
                  OutTextY(ys, NombreAff);
               end;
            end;
            pointCourant := pointCourant + deltaAxe;
         until (pointCourant >= Maxi);
         canvas.pen.style := psSolid;
   end end;// traceAxeY

   procedure zeroX;
   var
      x1, y1: integer;
   begin
      with mondeX do
         WindowXY(0, 0, x1, y1);
      if (x1 > limiteCourbe.left) and (x1 < limiteCourbe.right) then
         Myline(x1, limiteCourbe.bottom, x1, limiteCourbe.top);
   end;{zeroX}

   procedure zeroY;
   var
      x1, y1: integer;
   begin with mondeY do begin
         windowXY(0, 0, x1, y1);
         if (y1 > limiteCourbe.Top) and (y1 < limiteCourbe.Bottom) then
            Myline(limiteCourbe.left, y1, limiteCourbe.right, y1);
   end end;{zeroY}

begin // drawAxis
with canvas do begin
      signeMargeX := -1;
      pen.style := psSolid;
      pen.Width := 1;
      pen.color := couleurAxe;
      font.color := CouleurAxe;
      yAxeX := limiteCourbe.bottom;
      xAxeY := limiteCourbe.left;
      if mondeX.avecAxe
         then traceAxeX
         else zeroX;
      if mondeY.avecAxe
         then traceAxeY
         else zeroY;
   end;
end; // DrawAxis

procedure Tgraphe.Tri;
// Tri Shell Meyer-Baudoin p 456 ; origine vecteur en 0 
var
   cle: double;
   i, j: integer;
   SauvePoint: double;
   SauvePointBis: double;
begin
   i := 1;
   while (i < NbrePoints) do begin
      sauvePoint := data.valeurY[i];
      sauvePointBis := dataBis.valeurY[i];
      cle := valeurX[i];
      j := pred(i);
      while (j >= 0) and (valeurX[j] > cle) do begin
         data.valeurY[succ(j)] := data.valeurY[j];
         dataBis.valeurY[succ(j)] := dataBis.valeurY[j];
         Dec(j);
      end;
      valeurX[succ(j)] := cle;
      data.valeurY[succ(j)] := sauvePoint;
      dataBis.valeurY[succ(j)] := sauvePointBis;
      Inc(i);
   end;
end;{tri}

Function Tmonde.NomAxe : strCourte;

Function GetPrefixe(P : shortInt) : char;
begin
   result := ' ';
   if (length(Unite)>1) and
      (Unite[1] in caracPrefixe) and
      (pos('mol',unite)<>1) then begin
   case Unite[1] of
	  'f' : P := P-15;
	  'p' : P := P-12;
	  'n' : P := P-9;
	  '' : P := P-6;
	  'm' : P := P-3;
	  'k' : P := P+3;
	  'M' : P := P+6;
	  'G' : P := P+9;
	  'T' : P := P+12;
	  'P' : P := P+15;
   end;
   Unite := copy(Unite,2,length(Unite));
   end;
   case P of
	  -15 : result := 'f';
	  -12 : result := 'p';
	  -9 : result := 'n';
	  -6 : result := '';
	  -3 : result := 'm';
	  0  : result := ' ';
      +3 : result := 'k';
      +6 : result := 'M';
      +9 : result := 'G';
      +12 : result := 'T';
      +15 : result := 'P';
    end;
end;

var N : integer;
begin
    if unite = '' then begin
       result := nom;
       exit;
    end;   
    N := round(log10(exposant));
    if N=0
       then result := Unite
       else result := GetPrefixe(N)+Unite;
    if result<>''
       then if pos('/',result)>0
           then result := nom+'('+result+')'
           else result := nom+'/'+result
       else result := nom;
end;

end.

