unit spectrocfg;

interface

uses Windows, Graphics, SysUtils, Forms, Classes, iniFiles,
   Controls, Dialogs, Math,
   adPort, jmmUtil, oomisc;

const
   LambdaMaxMax    = 1100;
   LambdaMinMin    = 190;
   deltatMinCinetique = 8;

{
4104 NOVASPEC III
4106 NOVASPEC +
6004 LIBRA S4 
6006 LIBRA S6
7004 WPA S800
7006 WPA S1200
}

type
   TspectroPhoto = class
      nom : strCourte;
      lmin,lmax : integer;
      code : strCourte;
      absMax : double;
   end;

   TprocFin = procedure of object;
   TprocLambdaAbs = procedure (Lambda : integer;abs : double) of object;
   TprocErreur = procedure (Erreur : shortString) of object;

   TspectroBiochrom = class(TObject)
      FOnFin:  TprocFin;
      FOnErreur:  TprocErreur;
      FavailLambdaAbs:  TprocLambdaAbs;
      FOnChange:  TprocFin;
      FlambdaMax: integer;
      FlambdaMin: integer;
      Fconnecte : boolean;
      avecLF : boolean;
      FlambdaCourante : integer;
      Fcode,Fnom : shortString;
      constructor Create;
      procedure LitIni;
      procedure EcritIni;
      procedure TriggerAvailStr(CP: TObject; Count: word);
      procedure TriggerAvailBlanc(CP: TObject; Count: word);
      procedure TriggerAvailSpectre(CP: TObject; Count: word);
      procedure setLambdaMin(const Value: integer);
      procedure setLambdaMax(const Value: integer);
      function getDureeCinetique : integer;
      function TestErreur(reponse : shortString) : boolean;
   public
      codeErreur: shortString;
      spectreBinaire : boolean;
      AbsMax: double;
      DeltatCinetique, NombreCinetique: word;
      ComNumber:     word;
      VoieSerie:     TApdComPort;
      chaineLue:     shortString;
      longueurStr:   word;
      MiniC, MaxiC:  double;
      NomC, UniteC:  strCourte;
      SignifC:       shortString;
      LambdaCinetique,LambdaBeer,LambdaCinetiqueBis: word;
      BeerActif:     boolean;
      AbsSpectre: array[LambdaMinMin..LambdaMaxMax] of double;
      spectroBusy : boolean;
      debutBusy : TdateTime;
      property dureeCinetique: integer Read getDureeCinetique;
      property lambdaMax: integer Read FlambdaMax Write setLambdaMax;
      property lambdaMin: integer Read FlambdaMin Write setLambdaMin;
      property nom: shortString Read Fnom;      
      property connecte: boolean read Fconnecte;
      function DemandeReponse(commande: shortString;timeOut : integer) : shortString;
      procedure EnvoieCommande(commande: shortString);
      function Absorbance(lambda : integer) : double;
      function Version : shortString;
      procedure Release;
      function Control : boolean;
      procedure LanceZero(onFin : TprocFin);
      procedure LanceSpectre(onFin : TprocFin;onAvail : TprocLambdaAbs);
      function FormatAbs(nombre: double): strCourte;
   end;

  TlisteSpectro = class(TList)
  protected
    function Get(Index: integer): TspectroPhoto;
    procedure Put(Index: integer; Item: TspectroPhoto);
  public
    function Add(Item: TspectroPhoto): integer;
    function Remove(Item: TspectroPhoto): integer;
    property Items[Index: integer]: TspectroPhoto Read Get Write Put; default;
    procedure Clear; override;
  end;

var
   spectroBiochrom: TspectroBiochrom;

implementation

var
    spectros : TlisteSpectro;

constructor TspectroBiochrom.Create;
begin
   inherited Create;
   AbsMax := 2.5;
   FLambdaMin := 330;
   FLambdaMax := 800;
   ComNumber := 1;
   DeltatCinetique := deltatMinCinetique;
   NombreCinetique := 60;
   LambdaCinetique := 500;
   LambdaCinetiqueBis := 500;
   LambdaBeer := 500;
   MiniC := 0;
   MaxiC := 0.01;
   nomC  := 'c';
   UniteC := 'mol/L';
   SignifC := stConcentration;
   BeerActif := True;
   litIni;
end;

function TspectroBiochrom.GetDureeCinetique : integer;
begin
   result := NombreCinetique * DeltatCinetique;
end;

procedure TspectroBiochrom.LitIni;
var
   ini: tMemIniFile;
   nomFichier : string;
begin
   NomFichier := changeFileExt(application.exename,'.INI');
   if not FileExists(NomFichier) then
      exit;
   Ini := TMemIniFile.Create(NomFichier);

   ComNumber := Ini.ReadInteger('Spectro', 'Com', 1);

   MiniC := Ini.ReadFloat('Beer', 'Mini', 0);
   MaxiC := Ini.ReadFloat('Beer', 'Maxi', 0.01);
   if MaxiC<0.001 then MaxiC := 0.001;
   NomC := Ini.ReadString('Beer', 'Nom', 'c');
   if NomC='' then NomC := 'c';
   UniteC := Ini.ReadString('Beer', 'Unite', 'mol/L');
   SignifC := Ini.ReadString('Beer', 'Signif', stConcentration);
   BeerActif := Ini.ReadBool('Beer', 'Actif', True);
   LambdaBeer := Ini.ReadInteger('Beer', 'Lambda', 500);

   DeltatCinetique := Ini.ReadInteger('Cinetique', 'Deltat', deltatMinCinetique);
   if deltatCinetique<deltatMinCinetique then
        deltatCinetique := deltatMinCinetique;
   NombreCinetique := Ini.ReadInteger('Cinetique', 'Nombre', 60);
   LambdaCinetique := Ini.ReadInteger('Cinetique', 'Lambda', 500);
   LambdaCinetiqueBis := Ini.ReadInteger('Cinetique', 'LambdaBis', 500);
   
   Ini.Free;
end;

procedure TspectroBiochrom.ecritIni;
var                                     
   Ini: TMemIniFile;
   nomFichier : string;
begin
   NomFichier := changeFileExt(application.exename,'.INI');
   Ini := TMemIniFile.Create(nomFichier);
   Ini.WriteInteger('Spectro', 'Com', ComNumber);

   Ini.WriteFloat('Beer', 'Mini', miniC);
   Ini.WriteFloat('Beer', 'Maxi', maxiC);
   Ini.WriteString('Beer', 'Nom', nomC);
   Ini.WriteString('Beer', 'Unite', unitec);
   Ini.WriteString('Beer', 'Signif', signifC);
   Ini.WriteBool('Beer', 'Actif', BeerActif);
   Ini.WriteInteger('Beer', 'Lambda', LambdaBeer);

   Ini.WriteInteger('Cinetique', 'Deltat', DeltatCinetique);
   Ini.WriteInteger('Cinetique', 'Duree', DureeCinetique);
   Ini.WriteInteger('Cinetique', 'Nombre', NombreCinetique);
   Ini.WriteInteger('Cinetique', 'Lambda', LambdaCinetique);
   Ini.WriteInteger('Cinetique', 'LambdaBis', LambdaCinetiqueBis);   

   Ini.UpdateFile;
   Ini.Free;
end;

function TspectroBiochrom.demandeReponse(commande: shortString;timeOut : integer): shortString;
var ET : eventTimer;
begin
   voieSerie.onTriggerAvail := triggerAvailStr;
   chaineLue := '';
   avecLF := false;
   VoieSerie.output := commande+crCR+crLF;
   newTimer(ET,timeOut);
   repeat
        sleep(10);
        voieSerie.processCommunications;
        Application.ProcessMessages;
   until avecLF or timerExpired(ET);
   result := chaineLue;
end;

procedure TspectroBiochrom.EnvoieCommande(commande: shortString);
begin
   VoieSerie.output := commande+crCR+crLF;
end;

procedure TspectroBiochrom.Release;
begin
   DemandeReponse('RELEASE',9); // 0,5 s
   Fconnecte := false;
   if assigned(fonChange) then fOnChange();
end;

function TspectroBiochrom.Control : boolean;
var reponse : shortString;
begin
   reponse := demandeReponse('CONTROL',9);  // 0,5 s
   result := testErreur(reponse);
   Fconnecte := result;
   if assigned(fonChange) then fOnChange();   
end;

procedure TspectroBiochrom.LanceZero(onFin : TprocFin);
begin
   if spectroBusy and (DureeSec(now-debutBusy)<30) then exit;
   spectroBusy := true;
   debutBusy := now;
   FonFin := onFin;
   chaineLue := '';
   voieSerie.onTriggerAvail := triggerAvailBlanc;
   EnvoieCommande('SETREF');
end;

procedure TspectroBiochrom.LanceSpectre(onFin : TprocFin;onAvail : TprocLambdaAbs);
begin
   if spectroBusy and (DureeSec(now-debutBusy)<30) then exit;
   FonFin := onFin;
   SpectroBusy := true;
   debutBusy := now;
   FAvailLambdaAbs := onAvail;
   chaineLue := '';   
   voieSerie.onTriggerAvail := triggerAvailSpectre;
   FlambdaCourante := LambdaMin;
   if spectreBinaire
      then EnvoieCommande('SCANBIN')
      else EnvoieCommande('SCANTXT')
    // test WPA  
   // 1 s avant confirmation ; 5 s d'acquisition
   // 5 s de lecture en texte 3 s en binaire
end;

function TspectroBiochrom.Absorbance(Lambda : integer) : double;
var posA : integer;
    debut,fin : integer;
    reponse : shortString;
begin
     reponse := demandeReponse('READA? '+intToStr(lambda),4*18); // 4 secondes
     Result := sansSignif;
     try
      posA := pos('E00', chaineLue); // dbut de chaine
      if posA = 0 then exit;
      debut := 4;
      while not (chaineLue[debut] in ['-','+','0'..'9','.']) do inc(debut);
      fin := debut;
      while chaineLue[fin] in ['+','-','0'..'9','.'] do inc(fin);
      chaineLue := copy(chaineLue, debut, fin-debut+1);
      result := StrToFloatWin(chaineLue);
   except
      Result := sansSignif
   end;
end;

function TspectroBiochrom.Version : shortString;
var debut,fin : integer;
    reponse : shortString;
    i : integer;
begin
   reponse := demandeReponse('ID?',9); // 0,5 s
   if (length(reponse)>3) and (Copy(reponse,1,3)='E00')
       then begin
          debut := 5;
          fin := 5;
          while (fin<length(reponse)) and
                (reponse[fin]<>' ') do inc(fin);
          Fcode := copy(reponse,debut,fin-debut);
          for i := 0 to pred(Spectros.count) do
              if Fcode=spectros[i].code then begin
                 Fnom := spectros[i].nom;
                 FlambdaMin := spectros[i].lmin;
                 FlambdaMax := spectros[i].lmax;
                 absMax := spectros[i].absMax;
              end;
       end
       else Fcode := '';
    result := Fcode;   
end;

procedure TspectroBiochrom.TriggerAvailStr(CP: TObject; Count: word);
var
   i: word;
   C: char;
begin
   for i := 1 to Count do begin
       C := voieSerie.getChar;
       case  C of
          crLF : avecLF := true;
          crCR : ;
          else chaineLue := chaineLue + C;
       end;
   end;
end;

procedure TspectroBiochrom.TriggerAvailBlanc(CP: TObject; Count: word);
var
   i: word;
   C: char;
begin
   for i := 1 to Count do begin
       C := voieSerie.getChar;
       case C of
          crLF : begin
             avecLF := true;
             if (chaineLue<>'') and
                (chaineLue[1]='E') then begin
                  testErreur(chaineLue);
                  spectroBusy := false;
                  if assigned(fonChange) then fOnChange();
                  if assigned(FonFin) then FonFin();
             end;
             chaineLue := '';
          end;
          crCR : ;
          else chaineLue := chaineLue + C;
       end;
   end;
end;

procedure TspectroBiochrom.TriggerAvailSpectre(CP: TObject; Count: word);
var C: char;

procedure SpectreTxt;
var abso : double;
// sparateur CR LF
// fin du spectre E00 CR LF
begin
      case C of
           crCR : ;
           ' ' : ;
           crLF : begin
              if (chaineLue<>'') then
                if chaineLue[1]='E' then begin
                   testErreur(chaineLue);
                   SpectroBusy := false;
                   if assigned(fonChange) then fOnChange();
                   if assigned(FonFin) then FonFin();
                end
                else if pos('Scanning',chaineLue)>0 then begin
                   chaineLue := '';
                end
                else begin
                  try
                  abso := StrToFloatPoint(chaineLue);
                  chaineLue := '';
                  if assigned(FavailLambdaAbs) then
                     FavailLambdaAbs(FlambdaCourante,abso);
                  inc(FlambdaCourante);
                  except
                  end;
                end;
           end;
           else chaineLue := chaineLue + C;
      end;
end;

procedure SpectreBin;
var abso : double;
// dbut du spectre aprs scanning CR LF
// sparateur virgule
// fin du spectre E00 CR LF
begin
      case C of
           crCR : ;  // avant le LF
           ',' : if (chaineLue<>'') then begin
                  try
                  abso := StrToInt('$'+chaineLue);
                  if chaineLue[1]='F' then // complmentaire
                     abso := abso-4095;
                  abso := abso/1000;
                  chaineLue := '';
                  if assigned(FavailLambdaAbs) then
                     FavailLambdaAbs(FlambdaCourante,abso);
                  inc(FlambdaCourante);
                  except
                  end;
           end;
           crLF : begin
              if (chaineLue<>'') then
                if chaineLue[1]='E' then begin
                   testErreur(chaineLue);
                   SpectroBusy := false;
                   if assigned(fonChange) then fOnChange();
                   if assigned(FonFin) then FonFin();
                end
                else if pos('Scanning',chaineLue)>0 then begin
                   chaineLue := '';
                end;
           end;
           else chaineLue := chaineLue + C;
      end;
end;

var i: word;
begin
   for i := 1 to Count do begin
      C := voieSerie.getChar;
      if spectreBinaire then spectreBin else spectreTxt
   end;
end;

procedure TspectroBiochrom.setLambdaMin(const Value: integer);
begin
   if (Value >= lambdaMinMin) then
         FlambdaMin := Value;
end;

procedure TspectroBiochrom.setLambdaMax(const Value: integer);
begin
   if (Value <= lambdaMaxMax) then
          FlambdaMax := Value;
end;

function TspectroBiochrom.FormatAbs(Nombre: double): strCourte;
begin
   nombre := 0.001*round(nombre/0.001);
   result := FloatToStrF(nombre, ffGeneral, PrecisionMin, 0);
end;

const
NLignesDef = 59;
spectroDef : array[0..NLignesDef] of string[16] =
(
'[S4]',
'Nom=Libra S4',
'lmin=330',
'lmax=800',
'code=6004',
'absMax=2.5',
'[S6]',
'Nom=Libra S6',
'lmin=330',
'lmax=800',
'code=6006',
'absMax=2.5',
'[S11]',
'Nom=Libra S11',
'lmin=325',
'lmax=999',
'code=6011',
'absMax=3',
'[S12]',
'Nom=Libra S12',
'lmin=200',
'lmax=999',
'code=6012',
'absMax=3',
'[S21]',
'Nom=Libra S21',
'lmin=325',
'lmax=1100',
'code=6021',
'absMax=3',
'[S22]',
'Nom=Libra S22',
'lmin=190',
'lmax=1100',
'code=6022',
'absMax=3',
'[S800]',
'Nom=WPA S800',
'lmin=330',
'lmax=800',
'code=7004',
'absMax=2.5',
'[S1200]',
'Nom=WPA S1200',
'lmin=330',
'lmax=800',
'code=7006',
'absMax=2.5',
'[NOVA III]',
'Nom=NOVASPEC III',
'lmin=330',
'lmax=800',
'code=4104',
'absMax=2.5',
'[NOVA PLUS]',
'Nom=NOVASPEC +',
'lmin=330',
'lmax=800',
'code=4106',
'absMax=2.5'
);

Procedure ChargeListeSpectro;
var fichier : TMemIniFile;
    NomFichier : string;
    i : integer;
    liste : TstringList;
    listeSpectro : TstringList;
    aspectro : TspectroPhoto;
    section : shortString;
begin
    NomFichier := changeFileExt(application.exeName,'.ini');
    fichier := TMemIniFile.create(nomFichier);
    if not FileExists(NomFichier) then begin
// si vide cration  l'aide de defaut
        liste := TstringList.create;
        for i := 0 to NlignesDef do
           liste.add(SpectroDef[i]);
        fichier.setStrings(liste);
        liste.Free;
    end;
    spectros := TlisteSpectro.create;
    listeSpectro:= TstringList.create;
    fichier.ReadSections(ListeSpectro);
    for i := 0 to pred(ListeSpectro.Count) do begin
        aspectro := TspectroPhoto.create;
        section := ListeSpectro[i];
        aspectro.code := fichier.ReadString(section,'code','');
        aspectro.lmin := fichier.ReadInteger(section,'lmin',330);
        aspectro.lmax := fichier.ReadInteger(section,'lmax',800);
        aspectro.nom := fichier.ReadString(section,'nom','');
        aspectro.absMax := fichier.ReadFloat(section,'absMax',2);
        spectros.Add(aspectro);
    end;
    fichier.free;
end; { ChargeListeSpectro }

function TlisteSpectro.Add(Item: TspectroPhoto): integer;
begin
  Result := inherited Add(Item);
end;

function TlisteSpectro.Get(Index: integer): TspectroPhoto;
begin
  Result := TspectroPhoto(inherited Get(Index));
end;

procedure TlisteSpectro.Put(Index: integer; Item: TspectroPhoto);
begin
  inherited Put(Index, Item);
end;

function TlisteSpectro.Remove(Item: TspectroPhoto): integer;
begin
  Result := inherited Remove(Item);
  Item.Free;
end;

procedure TlisteSpectro.Clear;
var
  i: integer;
begin
  for i := 0 to pred(Count) do
    items[i].Free;
  inherited Clear;
end;

function TspectroBiochrom.testErreur(reponse : shortString) : boolean;
var code : integer;
begin
   result := reponse='E00';
   codeErreur := '';
   try
   code := strToInt(copy(reponse,2,2));
   case code of
        0 : codeErreur := '';
        1 : codeErreur := 'Erreur de paramtre';
        2 : codeErreur := 'Problme de lampe UV';
        3 : codeErreur := 'Problme de lampe visible';
        4 : codeErreur := 'Erreur de longueur d''onde';
        5 : codeErreur := 'Erreur interne du spectro';
        90 : codeErreur := 'Erreur nombre de paramtres';
        91 : codeErreur := 'Attendre la fin de l''talonnage';
        92 : codeErreur := 'Commande non prvue';
        93 : codeErreur := 'Appuyer sur la touche OK (F2) du spectro';
        94 : codeErreur := 'Argument invalide';
        95 : codeErreur := 'Commande trop longue';
        98 : codeErreur := 'Attendre la fin de l''talonnage';
        99 : codeErreur := 'Erreur inconnue';
        else codeErreur := 'Problme '+reponse;
   end;
   except
      codeErreur := 'Problme '+reponse;
   end;
   if assigned(fonErreur) then fOnErreur(codeErreur);   
end;

initialization
   spectroBiochrom := TspectroBiochrom.Create;
   chargeListeSpectro;

finalization
   spectroBiochrom.ecritIni;
   spectroBiochrom.Free;
   spectros.free;
end.

